<?php

namespace Drupal\elevenlabs_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;

/**
 * Plugin implementation of the 'elevenlabs_default' formatter.
 *
 * @FieldFormatter(
 *   id = "elevenlabs_default",
 *   label = @Translation("Default"),
 *   field_types = {"elevenlabs"}
 * )
 */
class ElevenLabsGenerationDefaultFormatter extends ElevenLabsFormatterBase {

  protected static $elevenLabsIdPrefix = 'elevenlabs-id-';
  protected static $elevenLabsItemIdPrefix = 'elevenlabs-audio-';

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'show_speaker_label' => FALSE,
      'play_button_type' => 'audio',
      'global_play_button_enabled' => TRUE,
      'global_play_button_text' => 'Play',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();

    $element['global_play_button_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Global Play Button Enabled'),
      '#default_value' => $settings['global_play_button_enabled'],
    ];

    $element['global_play_button_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Global Play Button Text'),
      '#default_value' => $settings['global_play_button_text'],
    ];

    $element['show_speaker_label'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Speaker Label'),
      '#default_value' => $settings['show_speaker_label'],
    ];

    $element['play_button_type'] = [
      '#type' => 'select',
      '#options' => [
        'audio' => $this->t('Normal Audio Element'),
        'button' => $this->t('Play Button'),
        'hide' => $this->t('Hide'),
      ],
      '#title' => $this->t('Play Button Type'),
      '#default_value' => $settings['play_button_type'],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $settings = $this->getSettings();
    $summary[] = $this->t("Show Speaker Label: @show_speaker_label, Play Button Type: @play_button_type", [
      '@show_speaker_label' => $settings['show_speaker_label'] ? 'true' : 'false',
      '@play_button_type' => $settings['play_button_type'],
    ]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    $id = self::$elevenLabsIdPrefix . str_replace('_', '-', $this->fieldDefinition->getName());
    $renderedItems = [];
    $drupalSettings = [
      'currentTime' => 0,
      'state' => 'loading',
      'totalItems' => count($items),
    ];
    foreach ($items as $item) {
      $renderedItems[] = $this->preprocessItem($item);
      $drupalSettings['items'][] = $this->createDrupalSettings($item);
    }
    return [
      '#attached' => [
        'library' => [
          'elevenlabs_field/elevenlabs'
        ],
        'drupalSettings' => [
          'elevenlabs' => [
            $id => $drupalSettings,
          ],
        ],
      ],
      '#theme' => 'elevenlabs_field',
      '#speeches' => $renderedItems,
      '#attributes' => [
        'class' => [
          'elevenlabs-global-play',
        ],
        'id' => [
          $id,
        ],
        'disabled' => 'disabled',
      ],
      '#global_play_button_text' => $this->t($this->settings['global_play_button_text']),
      '#global_play_button_enabled' => FALSE, //$this->settings['global_play_button_enabled'],
      '#player_enabled' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {
      if ($item->text) {
        $element[$delta]['text'] = [
          '#type' => 'item',
          '#title' => $this->t('Text'),
          '#markup' => $item->text,
        ];
      }

      if ($item->speaker) {
        $element[$delta]['speaker'] = [
          '#type' => 'item',
          '#title' => $this->t('Speaker'),
          '#markup' => $speakers[$item->speaker] ?? $item->speaker,
        ];
      }

      if ($item->playing_time) {
        $element[$delta]['playing_time'] = [
          '#type' => 'item',
          '#title' => $this->t('Playing Time'),
          '#markup' => $item->playing_time,
        ];
      }

    }

    return $element;
  }

  private function createDrupalSettings($item) {
    return [
      'id' => self::$elevenLabsItemIdPrefix . $item->history_item_id,
      'playing_time' => $item->playing_time,
    ];
  }

  private function preprocessItem($item) {
    $speakers = $this->getSpeakers();
    $file = File::load($item->target_id);
    return [
      '#theme' => 'elevenlabs_audio',
      '#file_url' => $file ? \Drupal::service('file_url_generator')->generate($file->getFileUri())->toString() : 'not found',
      '#text' => $item->text,
      '#voice' => $speakers[$item->speaker] ?? $item->speaker,
      '#classes' => '',
      '#show_voice' => TRUE,
      '#show_text' => TRUE,
      '#play_button_type' => $this->settings['play_button_type'] ?? 'audio',
      '#attributes' => [
        'id' => self::$elevenLabsItemIdPrefix . $item->history_item_id,
      ],
    ];
  }

}
