<?php

namespace Drupal\elevenlabs_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\elevenlabs_field\ElevenLabsApiService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\ConstraintViolationInterface;

/**
 * Defines the 'elevenlabs' field widget.
 *
 * @FieldWidget(
 *   id = "elevenlabs",
 *   label = @Translation("ElevenLabs Generation"),
 *   field_types = {"elevenlabs"},
 * )
 */
class ElevenLabsGenerationWidget extends WidgetBase {

  /**
   * ElevenLabs voices.
   */
  public array $voices = [];

  /**
   * ElevenLabs API.
   */
  protected ElevenLabsApiService $elevenLabsApi;

  /**
   * Constructs a WidgetBase object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    ElevenLabsApiService $elevenLabsApi
  )
  {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->elevenLabsApi = $elevenLabsApi;
    $this->voices = $this->elevenLabsApi->getVoices();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('elevenlabs_field.api_service'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return ['foo' => 'bar'] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $settings = $this->getSettings();
    $element['foo'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Foo'),
      '#default_value' => $settings['foo'],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $settings = $this->getSettings();
    $summary[] = $this->t('Foo: @foo', ['@foo' => $settings['foo']]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Text'),
      '#default_value' => isset($items[$delta]->text) ? $items[$delta]->text : NULL,
      '#required' => FALSE,
    ];


    $element['speaker'] = [
      '#type' => 'select',
      '#title' => $this->t('Speaker'),
      '#options' => $this->getVoicesOptionsArray(),
      '#default_value' => isset($items[$delta]->speaker) ? $items[$delta]->speaker : NULL,
    ];

    $element['start_time'] = [
      '#type' => 'number',
      '#title' => $this->t('Start Time (ms)'),
      '#description' => $this->t('The start time in relations to the last speaker. Can be negative if they should speak over each other. Leave empty if it should just start directly after.'),
      '#default_value' => isset($items[$delta]->start_time) ? $items[$delta]->start_time : 0,
    ];

    $element['playing_time'] = [
      '#type' => 'hidden',
      '#title' => $this->t('Playing Time (ms)'),
      '#default_value' => isset($items[$delta]->playing_time) ? $items[$delta]->playing_time : NULL,
    ];

    /*$element['generate'] = [
      '#type' => 'button',
      '#value' => $this->t('Regenerate audio'),
    ];*/

    $element['history_item_id'] = [
      '#type' => 'hidden',
      '#default_value' => isset($items[$delta]->history_item_id) ? $items[$delta]->history_item_id : "",
    ];

    $element['target_id'] = [
      '#type' => 'hidden',
      '#default_value' => isset($items[$delta]->target_id) ? $items[$delta]->target_id : NULL,
    ];

    $element['#theme_wrappers'] = ['container', 'form_element'];
    $element['#attributes']['class'][] = 'elevenlabs-elements';
    $element['#attached']['library'][] = 'elevenlabs_field/elevenlabs';

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function errorElement(array $element, ConstraintViolationInterface $violation, array $form, FormStateInterface $form_state) {
    return isset($violation->arrayPropertyPath[0]) ? $element[$violation->arrayPropertyPath[0]] : $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $delta => $value) {
      if ($value['text'] === '') {
        $values[$delta]['text'] = NULL;
      }
      if ($value['speaker'] === '') {
        $values[$delta]['speaker'] = NULL;
      }
      if ($value['playing_time'] === '') {
        $values[$delta]['playing_time'] = NULL;
      }
    }
    return $values;
  }

  /**
   * Undocumented function
   *
   * @return void
   */
  private function getVoicesOptionsArray() {
    $options[''] = $this->t('- Select a voice -');
    foreach ($this->voices['voices'] as $voice) {
      $options[$voice['voice_id']] = $voice['name'];
    }
    return $options;
  }

}
